import ddf.minim.*; //<>//

/*********************************************************************************/
/* オセロ風リバーシゲーム                                                        */
/*-------------------------------------------------------------------------------*/
/* 大人の約束                                                                    */
/* 本プログラム（以下、本APと呼称）の著作権はMSLABOにあります。本APの全部または  */
/* 一部を、無断で配布・掲載・盗用・流用することを禁止します。                    */
/* 本APは学習目的に作成されています。無償で利用可能ですが、どのような目的にせよ、*/
/* 利用によって発生するいかなる事象もMSLABOは責任を負いません。必ず自己責任で利用*/
/* してください。                                                                */
/*                                                                               */
/* 本APに使われている素材（画像、音楽、文字）は下記に権利があります。利用にあた  */
/* っては、必ず各権利者様の指示または関連HPの記載に従ってください。              */
/* 特に音楽素材は、無断での再配布は禁止です（私は今回の配布に際し、権利者様より  */
/* 許可を得ております）。勝手に再配布しないようにお願いします。                  */
/*                                                                               */
/* 画像                                                                          */
/* ・GAHAG 著作権フリー写真・イラスト素材集 様                                   */
/*   URL : http://gahag.net/                                                     */
/* 音楽                                                                          */
/* ・On-Jin音人 様                                                               */
/*   URL : http://on-jin.com/                                                    */
/* 文字                                                                          */
/* ・M+FONTS 様                                                                  */
/*   URL : http://mplus-fonts.osdn.jp/about.html                                 */
/*                                                                               */
/* また配布ファイルには含まれていませんが、ゲーム画面に以下の文字を利用させてい  */
/* ただいております。アンニャントロマン文字の権利は下記の通りです。              */
/* ・森の中の猫の小屋 様                                                         */
/*   URL : http://inatsuka.com/                                                  */
/*                                                                               */
/* MSLABOの関連HPは下記の通りです。ソースコードの解説、最新版の入手、不具合のご報*/
/* 告やご意見などについては、下記URLを参照してください。                         */
/* なお本APの不具合や機能について、各素材サイト様へのお問い合わせはご遠慮くださ  */
/* い。本APについての問い合わせは、下記URLまでお願いいたします。                 */
/*                                                                               */
/* 問い合わせ先：                                                                */
/* URL : http://mslabo.sakura.ne.jp/WordPress/                                   */
/*                                                                               */
/* 本プログラムが、みなさまのPROCESSING学習の一助になれば幸いです。              */
/*                                                                               */
/*********************************************************************************/
/************************************************/
/* 定数                                         */
/************************************************/
final String   BORD_IMGNAME = "board.png";
final String   WHITE_PIECE_IMGNAME = "white.png";
final String   BLACK_PIECE_IMGNAME = "black.png";
final String   FONT_NAME = "mplus-1p-regular-48.vlw";
final String   DAON_NAME = "spo_ge_igo_utu01.mp3";
final int      WHITE    = 0;
final int      BLACK    = 1;
final int      SPACE    = -1;
final int      FONT_SIZE = 32;


/************************************************/
/* グローバル変数                              */
/************************************************/
PImage    boardImg;         // ボード画像
PImage    blackImg;         // 黒駒画像
PImage    whiteImg;         // 白駒画像
PFont    font;              //文字
int      turnColor;         // 手番の色
int      pieces[];          // 駒配列
int      blackPiecesCount;  // 黒の駒数
int      whitePiecesCount;  // 白の駒数
boolean    gameOver;        //GAMEOVERフラグ
Minim     minim;            //効果音用
AudioSample kick;           //効果音用

/************************************************/
/* 初期処理関数                                 */
/************************************************/
public void setup() {
  size( 1096, 720 );
  
  // 各画像を読み込む
  boardImg = loadImage( BORD_IMGNAME );
  blackImg = loadImage( BLACK_PIECE_IMGNAME );
  whiteImg = loadImage( WHITE_PIECE_IMGNAME );

  //文字を読み込む
  font = loadFont( FONT_NAME );
  textFont( font, FONT_SIZE );
  textAlign( LEFT, TOP );
  
  //効果音を読み込む
  minim = new Minim( this );
  kick = minim.loadSample( DAON_NAME );

  //駒を記憶する配列を準備する
  pieces = new int[64];

  // ゲームの初期化を行う
  initGame();
}

/*----------------------------------------------*/
/* ゲーム初期化関数                             */
/*----------------------------------------------*/
void initGame(){
  //黒の手番から始まる
  turnColor = BLACK;

  //駒情報をすべて初期化する
  for( int i = 0; i < 64; i++ ){
    pieces[ i ] = SPACE;
  }

  //黒白の駒数を初期化する
  blackPiecesCount = 0;
  whitePiecesCount = 0;

  //中央に白黒の駒を並べる
  pieces[ 27 ] = BLACK;
  pieces[ 28 ] = WHITE;
  pieces[ 35 ] = WHITE;
  pieces[ 36 ] = BLACK;

  //ゲーム状態変数をPLAY中にする
  gameOver = false;;
}

/************************************************/
/* 描画処理関数                                 */
/************************************************/
public void draw() {
  // 背景を表示する
  background( boardImg );
  
  //駒の数を表示する
  piecesCountDisp();
   
  //駒を表示する
  piecesDisp();
  
  //GAME OVER検査
  if( ( blackPiecesCount + whitePiecesCount ) > 63 ){
    //すべてのマスが埋まったのでGAME終了
    gameOver = true;
    dispGameOver();
  } else {
    //手番を表示する
    turnDisp();
  }
}

/*----------------------------------------------------------*/
/* 盤上の黒と白の駒数を数える関数                           */
/*----------------------------------------------------------*/
void  piecesCountDisp(){
  
  whitePiecesCount = 0;
  blackPiecesCount = 0;

  for( int index = 0; index < 64; index++ ){
    //それぞれの色を数える
    if( pieces[ index ] == WHITE ){
      whitePiecesCount++;
    } else if( pieces[ index ] == BLACK ){
      blackPiecesCount++;
    }
  }
  
  String    msg;

  //文字の色を黒にする
  fill( 0 );
  
  //黒駒数を表示する
  msg = "黒の数は" + blackPiecesCount + "個";
  textDisp( msg, 800, 235, 8 );

  //白駒数を表示する
  msg = "白の数は" + whitePiecesCount + "個";
  textDisp( msg, 800, 320, 8 );
  return;
}

/*------------------------------------------------------*/
/* メッセージを表示する関数                             */
/*------------------------------------------------------*/
void textDisp( String msg, int x, int y, int maxMojisuu ){
  int      dx, dy;
  int      dc;
  
  dx = x;    //1文字目の描画開始位置
  dy = y;    //〃
  dc = 1;    //置いた文字の数

  //先頭から１文字ずつ取り出して置いていく
  for( int i = 0; i < msg.length(); i++ ){
    //1文字取り出して描画
    String c = msg.substring( i, i+1 );
    text( c, dx, dy );
    dx = dx + FONT_SIZE;
    dc++;
    
    //MAXまで置いたら改行する
    if( dc > maxMojisuu ){
      dc = 1;
      dx = x;
      dy = dy + FONT_SIZE;
    }
  }

  return;
}

/*----------------------------------------------------------*/
/* 黒白のどちらの手番か表示する関数                         */
/*----------------------------------------------------------*/
void turnDisp(){
  
  //打ち先があるか調べる
  boolean putOk = checkAllPutOk() ;
  
  //もし打ち先がない場合はPASSする
  if( putOk == false ){
    if( turnColor == WHITE ){
      turnColor = BLACK;
    } else {
      turnColor = WHITE;
    }
  }
  
  String    msg;
  
  //文字の色を黒にする
  fill( 0 );
  
  if( turnColor == BLACK ){
    //黒の手番
    msg = "黒の番です。";
  } else {
    //白の手番
    msg = "白の番です。";
  }
  textDisp( msg, 695, 580, 12 );
}

/*----------------------------------------------------------*/
/* GAME OVERの通知を表示する関数                            */
/*----------------------------------------------------------*/
void dispGameOver(){
  String    msg;

  fill( color(50,50,255));
  if( blackPiecesCount > whitePiecesCount ){
    msg = "黒の勝ち。SPACEキーを押すと再開します。";
  } else if( blackPiecesCount < whitePiecesCount ){
    msg = "白の勝ち。SPACEキーを押すと再開します。";
  } else {
    msg = "引き分け。SPACEキーを押すと再開します。";
  }
  textDisp( msg, 695, 580, 12 );
}


/*----------------------------------------------------------*/
/* 盤上に、駒が置ける場所があるか検査する関数               */
/*----------------------------------------------------------*/
/* 戻り値                                                   */
/* true : 置ける    false : 置けない                        */
/*----------------------------------------------------------*/
boolean  checkAllPutOk(){
  
  return( true );
}

/*-------------------------------------------------------*/
/* 駒を描画する関数                                      */
/*-------------------------------------------------------*/
void  piecesDisp(){
  return;
}